@@ -21,7 +21,7 @@ appoptics_apm_noop defines no-op classes for platforms we don't support building
 import threading
 
 
-class Metadata(object):
+class Metadata:
     def __init__(self, _=None):
         pass
 
@@ -49,7 +49,7 @@ class Metadata(object):
         return ''
 
 
-class Context(object):
+class Context:
     transaction_dict = threading.local()
 
     @staticmethod
@@ -117,7 +117,7 @@ class Context(object):
         return 0
 
 
-class Event(object):
+class Event:
     def __init__(self, _=None, __=None):
         pass
 
@@ -144,7 +144,7 @@ class Event(object):
         return Event()
 
 
-class Reporter(object):
+class Reporter:
     def __init__(self, *args, **kwargs):
         self.init_status = 0
 
@@ -154,8 +154,11 @@ class Reporter(object):
     def sendStatus(self, _, __=None):
         pass
 
+    def flush(self):
+        pass
 
-class Span(object):
+
+class Span:
     @staticmethod
     def createHttpSpan(_, __, ___, ____, _____, ______, _______):
         pass
@@ -165,7 +168,7 @@ class Span(object):
         pass
 
 
-class MetricTags(object):
+class MetricTags:
     def __init__(self, count):
         super(MetricTags, self).__init__()
 
@@ -174,7 +177,7 @@ class MetricTags(object):
         pass
 
 
-class CustomMetrics(object):
+class CustomMetrics:
     @staticmethod
     def summary(*args, **kwargs):
         pass
@@ -184,7 +187,7 @@ class CustomMetrics(object):
         pass
 
 
-class Config(object):
+class Config:
     @staticmethod
     def getVersionString():
         return "No extension loaded."
