@@ -15,8 +15,6 @@
 """
 import logging
 
-import six
-
 from appoptics_apm import util
 
 logger = logging.getLogger(__name__)
@@ -170,7 +168,7 @@ def profile_drop_collection(func, args, kwargs, func_results):
     (db, name_or_collection) = args[:2]
     report_kvs = {}
 
-    if isinstance(name_or_collection, six.string_types):
+    if isinstance(name_or_collection, str):
         collection_name = name_or_collection
     else:
         collection_name = name_or_collection.name
@@ -187,7 +185,7 @@ def profile_command(func, args, kwargs, func_results):
     report_kvs = {}
 
     _add_connection_info(report_kvs, db)
-    if not isinstance(command, six.string_types):
+    if not isinstance(command, str):
         command = _to_json(command)
 
     report_kvs['Command'] = command
