@@ -190,7 +190,7 @@ def wrap_send_packed_command(layer_name, func):
     This relies on the module internals, and just sends an info event when this
     function is called.
     """
-    @wraps(func)  # XXX Not Python2.4-friendly
+    @wraps(func)
     def wrapper(*f_args, **f_kwargs):
         ret = func(*f_args, **f_kwargs)
         try:
