@@ -16,11 +16,11 @@
 Checks appoptics_apm.config['inst_enabled'] and imports as requested. used by middleware
 and djangoware.
 """
-from appoptics_apm import config
+import appoptics_apm
 
 
 def _enabled(m):
-    return config['inst_enabled'][m]
+    return appoptics_apm.config['inst_enabled'][m]
 
 
 def load_inst_modules():
