@@ -28,7 +28,6 @@ import logging
 import re
 from datetime import datetime
 
-import six
 from bson.code import Code
 from bson.dbref import DBRef
 from bson.objectid import ObjectId
@@ -38,19 +37,19 @@ logger = logging.getLogger(__name__)
 
 __all__ = ('skeleton', 'sanitize')
 
-BSON_TYPES = (
-    set(six.integer_types) | set(six.string_types)
-    | set([
-        bool,
-        float,
-        datetime,
-        ObjectId,
-        type(re.compile('')),
-        Code,
-        type(None),
-        DBRef,
-        SON,
-    ]))
+BSON_TYPES = set([
+    int,
+    str,
+    bool,
+    float,
+    datetime,
+    ObjectId,
+    type(re.compile('')),
+    Code,
+    type(None),
+    DBRef,
+    SON,
+])
 
 
 def skeleton(query_part, preserve_first=False, keep_values=False, no_wrap=False):
@@ -82,9 +81,9 @@ def skeleton(query_part, preserve_first=False, keep_values=False, no_wrap=False)
             if sub is not None:
                 out.append(sub)
         if no_wrap:
-            return six.u(','.join(out))
+            return ','.join(out)
         else:
-            return six.u('[{o}]'.format(o=','.join(out)))
+            return '[{o}]'.format(o=','.join(out))
     elif t == dict:
         out = []
         for key in sorted(query_part.keys()):
@@ -94,9 +93,9 @@ def skeleton(query_part, preserve_first=False, keep_values=False, no_wrap=False)
             else:
                 out.append(key)
         if no_wrap:
-            return six.u(','.join(out))
+            return ','.join(out)
         else:
-            return six.u('{%s}' % ','.join(out))
+            return '{%s}' % ','.join(out)
     elif keep_values:
         return str(query_part)
     elif t not in BSON_TYPES:
