@@ -10,7 +10,7 @@
 
 /////// Metatdata ///////
 
-Metadata::Metadata(oboe_metadata_t *md) {
+Metadata::Metadata(const oboe_metadata_t *md) {
     oboe_metadata_copy(this, md);
 }
 
@@ -80,9 +80,9 @@ void Context::setDefaultSampleRate(int newRate) {
 }
 
 void Context::getDecisions(
-    // this paramter list is too long, but other forms of sending info 
+    // this paramter list is too long, but other forms of sending info
     // between python/ruby and c++ would require extra roudn trips
-    // and may be less efficient. 
+    // and may be less efficient.
     // TODO: benchmark this assumption
 
     // output
@@ -214,6 +214,10 @@ int Context::isReady(unsigned int timeout) {
     return oboe_is_ready(timeout);
 }
 
+bool Context::isLambda() {
+    return (bool) oboe_is_lambda();
+}
+
 /**
  * Create a new event object that continues the trace context.
  *
@@ -289,7 +293,7 @@ bool Event::addInfo(char *key, bool val) {
 }
 
 /*
- * this function was added for profiling 
+ * this function was added for profiling
  * to report the timestamps of omitted snapshots
  */
 bool Event::addInfo(char *key, const long *vals, int num) {
@@ -324,7 +328,7 @@ bool Event::addInfo(char *key, const std::vector<FrameData> &vals) {
             oboe_bson_append_string(&(this->bbuf), "C", (val.klass).c_str());
         if (val.file != "")
             oboe_bson_append_string(&(this->bbuf), "F", (val.file).c_str());
-        if (val.lineno != 0)
+        if (val.lineno > 0)
             oboe_bson_append_long(&(this->bbuf), "L", (int64_t)val.lineno);
 
         oboe_bson_append_finish_object(&(this->bbuf));
@@ -357,7 +361,7 @@ bool Event::addHostname() {
     return oboe_event_add_info(this, "Hostname", oboe_hostname) == 0;
 }
 
-bool Event::addContextOpId(oboe_metadata_t *md) {
+bool Event::addContextOpId(const oboe_metadata_t *md) {
     char buf[OBOE_MAX_METADATA_PACK_LEN]; // Flawfinder: ignore
     oboe_metadata_tostr(md, buf, OBOE_MAX_METADATA_PACK_LEN);
     buf[58] = '\0';
@@ -365,7 +369,7 @@ bool Event::addContextOpId(oboe_metadata_t *md) {
     return oboe_event_add_info(this, "ContextOpId", &buf[42]);
 }
 
-bool Event::addSpanRef(oboe_metadata_t *md) {
+bool Event::addSpanRef(const oboe_metadata_t *md) {
     char buf[OBOE_MAX_METADATA_PACK_LEN]; // Flawfinder: ignore
     oboe_metadata_tostr(md, buf, OBOE_MAX_METADATA_PACK_LEN);
     buf[58] = '\0';
@@ -483,18 +487,26 @@ std::string Span::createHttpSpan(const char *transaction, const char *url, const
 
 MetricTags::MetricTags(size_t count) {
     tags = new oboe_metric_tag_t[count];
+    for (size_t i = 0; i < count; i++) {
+        tags[i].key = nullptr;
+        tags[i].value = nullptr;
+    }
     size = count;
 }
 
 MetricTags::~MetricTags() {
+    for (size_t i = 0; i < size; i++) {
+        delete tags[i].key; tags[i].key = nullptr;
+        delete tags[i].value; tags[i].value = nullptr;
+    }
     delete[] tags;
 }
 
-bool MetricTags::add(size_t index, char *k, char *v) {
+bool MetricTags::add(size_t index, const char *k, const char *v) {
     if (index < size) {
-        tags[index].key = k;
-        tags[index].value = v;
-        return true;
+        tags[index].key = strdup(k);
+        tags[index].value = strdup(v);
+        return (tags[index].key != nullptr && tags[index].value != nullptr);
     }
     return false;
 }
@@ -520,6 +532,15 @@ int CustomMetrics::increment(const char *name, const int count, const int host_t
     return oboe_custom_metric_increment(name, count, host_tag, service_name, tags->get(), tags_count);
 }
 
+/////// Profiling ///////
+
+// adding this to have a proper interface, even though it just
+// returns the return value of the oboe function
+// 0 indicates not to profile, returns -1 if the collector hasn't sent anything
+int OboeProfiling::get_interval() {
+    return oboe_get_profiling_interval();
+}
+
 /////// Reporter ///////
 
 Reporter::Reporter(
@@ -544,12 +565,15 @@ Reporter::Reporter(
     double token_bucket_rate,       // custom token bucket rate
     int file_single,                // use single files in file reporter for each event
 
-    int ec2_metadata_timeout,  // the timeout (milli seconds) for retrieving EC2 metadata
-    std::string grpc_proxy     // HTTP proxy address and port to be used for the gRPC connection
+    int ec2_metadata_timeout,       // the timeout (milli seconds) for retrieving EC2 metadata
+    std::string grpc_proxy,         // HTTP proxy address and port to be used for the gRPC connection
+    int stdout_clear_nonblocking,   // flag indicating if the O_NONBLOCK flag on stdout should be cleared,
+                                    // only used in lambda reporter (off=0, on=1, default off)
+    int is_grpc_clean_hack_enabled  // flag indicating if custom grpc clean hack enabled (default 0)
 ) {
     oboe_init_options_t options;
     memset(&options, 0, sizeof(options));
-    options.version = 10;
+    options.version = 12;
     oboe_init_options_set_defaults(&options);
 
     if (hostname_alias != "") {
@@ -583,6 +607,8 @@ Reporter::Reporter(
     if (grpc_proxy != "") {
         options.proxy = grpc_proxy.c_str();
     }
+    options.stdout_clear_nonblocking = stdout_clear_nonblocking;
+    options.is_grpc_clean_hack_enabled = is_grpc_clean_hack_enabled;
     init_status = oboe_init(&options);
 }
 
