@@ -21,7 +21,7 @@ import appoptics_apm
 logger = logging.getLogger(__name__)
 
 
-class UrlGetter(object):
+class UrlGetter:
     """Base class which provides a common interface to retrieve the request URL from an instrumented framework. In case
     of complex/ time-consuming URL retrieval mechanisms, a derived class should be created which overrides set_url."""
     def __init__(self, url=None):
@@ -45,7 +45,7 @@ class UrlGetter(object):
         return self._url
 
 
-class Rule(object):
+class Rule:
     """Abstract base class for all filter rules."""
     def __init__(self, tracing_mode):
         self.tracing_mode = tracing_mode
@@ -80,7 +80,7 @@ class ExtensionsRule(Rule):
         return url.endswith(self._extensions)
 
 
-class UrlFilter(object):
+class UrlFilter:
     """URL filter consisting of a list of filter rules for custom URL filtering.
     A valid filter rule must be provided in form of a dictionary having a structure as outlined below:
     [0]
