@@ -14,6 +14,7 @@
 """ AppOptics APM instrumentation API for Python.
 When imported, the appoptics_apm_init() function will be executed to do some initialization tasks.
 """
+import configparser
 import inspect
 import logging
 import os
@@ -26,7 +27,6 @@ import types
 from collections import defaultdict, namedtuple
 from functools import reduce
 
-import six
 from decorator import decorator
 
 import appoptics_apm
@@ -42,7 +42,7 @@ from appoptics_apm.version import __version__
 AGENT_START_TIME = time.time() * 1e6
 
 
-class OboeTracingMode(object):
+class OboeTracingMode:
     """Provides an interface to translate the string representation of tracing_mode to the C-extension equivalent."""
     OBOE_SETTINGS_UNSET = -1
     OBOE_TRACE_DISABLED = 0
@@ -75,24 +75,25 @@ ZERO_SAMPLE_RATE_MASK = 0b1111000000000000000000000000
 ZERO_SAMPLE_SOURCE_MASK = 0b0000111111111111111111111111
 
 
-def _apm_str(non_string):
-    """converts non-string type to string if possible
-       otherwise return as is intact
-       * this typically happens python2 unicode string
-    """
-    try:
-        return str(non_string)
-    except UnicodeEncodeError:
-        # 'utf-8 encoding is more safe if there is regex processing etc. for
-        # consistence encode unicode using utf-8
-        return non_string.encode('utf-8')
-    except Exception:
-        return non_string
+def is_lambda():
+    "Checks if agent is running in an AWS Lambda environment." ""
+    if os.environ.get('AWS_LAMBDA_FUNCTION_NAME') and os.environ.get("LAMBDA_TASK_ROOT"):
+        return True
+    return False
 
 
 util_logger = logging.getLogger(__name__)
 reporter_instance = None
+_agent_enabled = True
 try:
+    if os.environ.get('APPOPTICS_AGENT_ENABLED', 'true').lower() == 'false':
+        _agent_enabled = False
+        util_logger.info(
+            "AppOptics agent is disabled and will not report any traces because the environment variable "
+            "APPOPTICS_AGENT_ENABLED is set to 'false'! If this is not intended either unset the variable or set it to "
+            "a value other than false. Note that the value of APPOPTICS_AGENT_ENABLED is case-insensitive.")
+        raise ImportError
+
     from appoptics_apm.swig.oboe import (
         Context as SwigContext,
         Event as SwigEvent,
@@ -103,6 +104,10 @@ try:
         MetricTags,
         Config as SwigConfig,
     )
+    if not os.environ.get('APPOPTICS_SERVICE_KEY', None) and not is_lambda():
+        util_logger.error("Missing service key. Tracing disabled.")
+        _agent_enabled = False
+        raise ImportError
 except ImportError as e:
     try:
         from appoptics_apm.appoptics_apm_noop import (
@@ -116,23 +121,30 @@ except ImportError as e:
             Config as SwigConfig,
         )
 
-        if sys.platform.startswith('linux'):
-            util_logger.warning(
-                """Missing extension library.
-                           Tracing is disabled and will go into no-op mode.
-                           Contact support@appoptics.com if this is unexpected.
-                           Error: %s
-                           See: https://docs.appoptics.com/kb/apm_tracing/python/""" % e)
-        else:
-            util_logger.warning(
-                """Platform %s not yet supported.
-                           See: https://docs.appoptics.com/kb/apm_tracing/supported_platforms/
-                           Tracing is disabled and will go into no-op mode.
-                           Contact support@appoptics.com if this is unexpected.""" % sys.platform)
+        if _agent_enabled:
+            # only log the following messages if agent wasn't explicitly disabled via APPOPTICS_AGENT_ENABLED or due to
+            # missing service key
+            if sys.platform.startswith('linux'):
+                util_logger.warning(
+                    """Missing extension library.
+                               Tracing is disabled and will go into no-op mode.
+                               Contact support@appoptics.com if this is unexpected.
+                               Error: %s
+                               See: https://docs.appoptics.com/kb/apm_tracing/python/""" % e)
+            else:
+                util_logger.warning(
+                    """Platform %s not yet supported.
+                               See: https://docs.appoptics.com/kb/apm_tracing/supported_platforms/
+                               Tracing is disabled and will go into no-op mode.
+                               Contact support@appoptics.com if this is unexpected.""" % sys.platform)
     except ImportError as err:
         util_logger.error(
             """Unexpected error: %s.
                      Please reinstall or contact support@appoptics.com.""" % err)
+    finally:
+        # regardless of how we got into this (outer) exception block, the agent will not be able to trace (and thus be
+        # disabled)
+        _agent_enabled = False
 
 
 def custom_metrics_summary(name, value, count=1, host_tag=False, service_name=None, tags=None, tags_count=0):
@@ -219,7 +231,7 @@ class AppOpticsApmException(Exception):
     """ AppOptics APM Exception Class """
 
 
-class AppOpticsApmConfig(object):
+class AppOpticsApmConfig:
     """ AppOptics APM Configuration Class
     The precedence: in-code keyword arguments > Environment Variables > config file > default values.
     Note that oboe doesn't read configurations by itself. The Python agent needs to
@@ -247,6 +259,7 @@ class AppOpticsApmConfig(object):
         # Update the config with default values
         self._config = {
             #'sample_rate' and 'tracing_mode' do not have any default values (i.e. they are unset by default)
+            'sample_rate': None,
             'tracing_mode': None,
             'collector': '',  # the collector address in host:port format.
             'reporter': '',  # the reporter mode, either 'udp' or 'ssl'.
@@ -272,7 +285,8 @@ class AppOpticsApmConfig(object):
             'log_trace_id': 'never',
             'proxy': '',
             'transaction': defaultdict(lambda: True),
-            'inst': defaultdict(lambda: True)
+            'inst': defaultdict(lambda: True),
+            'is_grpc_clean_hack_enabled': False,
         }
         self._config['transaction']['prepend_domain_name'] = False
         for inst in self.supported_instrumentations:
@@ -301,8 +315,13 @@ class AppOpticsApmConfig(object):
         val = os.environ.get('APPOPTICS_APM_PREPEND_DOMAIN_NAME', None)
         if val is not None:
             self._set_config_value('transaction.prepend_domain_name', val)
+        available_envvs = set(self._config.keys())
+        # bucket rate and capacity should be configurable through environment variable only under AWS Lambda
+        if not is_lambda():
+            available_envvs.remove('token_bucket_capacity')
+            available_envvs.remove('token_bucket_rate')
 
-        for key in self._config:
+        for key in available_envvs:
             if key in ('inst_enabled', 'transaction', 'inst'):
                 # we do not allow complex config options to be set via environment variables
                 continue
@@ -316,8 +335,8 @@ class AppOpticsApmConfig(object):
         self.delimiter, i.e., the position of the element to be changed in the nested dictionary)"""
         def _convert_to_bool(val):
             """Converts given value to boolean value"""
-            val = val.lower() if isinstance(val, six.string_types) else val
-            return val == 'true' if isinstance(val, six.string_types) and val in ('true', 'false') else bool(int(val))
+            val = val.lower() if isinstance(val, str) else val
+            return val == 'true' if isinstance(val, str) and val in ('true', 'false') else bool(int(val))
 
         # _config is a nested dict, thus find most deeply nested sub dict according to the provided keys
         # by defaulting to None in d.get(), we do not allow the creation of any new (key, value) pair, even
@@ -338,17 +357,28 @@ class AppOpticsApmConfig(object):
                 if not 0 <= sample_rate <= 1:
                     raise ValueError
                 self._config[key] = sample_rate
+                SwigContext.setDefaultSampleRate(int(sample_rate * 1e6))
             elif keys == ['ec2_metadata_timeout']:
                 timeout = int(val)
                 if timeout not in range(0, 3001):
                     raise ValueError
                 self._config[key] = timeout
+            elif keys == ['token_bucket_capacity']:
+                bucket_cap = float(val)
+                if not 0 <= bucket_cap <= 8.0:
+                    raise ValueError
+                self._config[key] = bucket_cap
+            elif keys == ['token_bucket_rate']:
+                bucket_rate = float(val)
+                if not 0 <= bucket_rate <= 4.0:
+                    raise ValueError
+                self._config[key] = bucket_rate
             elif keys == ['proxy']:
-                if not isinstance(val, six.string_types) or not val.startswith('http://'):
+                if not isinstance(val, str) or not val.startswith('http://'):
                     raise ValueError
                 self._config[key] = val
             elif keys == ['tracing_mode']:
-                if not isinstance(val, six.string_types):
+                if not isinstance(val, str):
                     raise ValueError
                 val = val.lower()
                 if val in ['always', 'never']:
@@ -362,7 +392,7 @@ class AppOpticsApmConfig(object):
                 self._config[key] = val
                 SwigContext.setTracingMode(OboeTracingMode.get_oboe_trace_mode(val))
             elif keys == ['reporter']:
-                if not isinstance(val, six.string_types) or val.lower() not in ['udp', 'ssl', 'null', 'file']:
+                if not isinstance(val, str) or val.lower() not in ('udp', 'ssl', 'null', 'file', 'lambda'):
                     raise ValueError
                 self._config[key] = val.lower()
             elif keys == ['debug_level']:
@@ -373,7 +403,7 @@ class AppOpticsApmConfig(object):
                 # update loggig level of agent util_logger
                 appoptics_apm.ao_logging.set_ao_log_level(val)
             elif keys == ['log_trace_id']:
-                if not isinstance(val, six.string_types) or val.lower() not in [
+                if not isinstance(val, str) or val.lower() not in [
                         'never',
                         'sampled',
                         'traced',
@@ -381,6 +411,8 @@ class AppOpticsApmConfig(object):
                 ]:
                     raise ValueError
                 self._config[key] = val.lower()
+            elif keys == ['is_grpc_clean_hack_enabled']:
+                self._config[key] = _convert_to_bool(val)
             elif (keys[0] == 'inst' and keys[1] in self.supported_instrumentations and keys[2] == 'collect_backtraces'):
                 self._config[keys[0]][keys[1]][keys[2]] = _convert_to_bool(val)
             elif isinstance(sub_dict, dict) and keys[-1] in sub_dict:
@@ -399,12 +431,12 @@ class AppOpticsApmConfig(object):
     def update_with_cnf_file(self, cnf_path):
         """Update the settings with the config file, if any."""
 
-        cnf = six.moves.configparser.ConfigParser()
+        cnf = configparser.ConfigParser()
         try:
             if not cnf.read(cnf_path):
                 util_logger.warning('Failed to open the config file: {}.'.format(cnf_path))
                 return
-        except six.moves.configparser.Error as e:
+        except configparser.Error as e:
             util_logger.warning('Failed to read or parse config file: {e}'.format(e=e))
 
         # make sure we are reading the main section first, such that the debug_level can be evaluated before any other
@@ -416,10 +448,7 @@ class AppOpticsApmConfig(object):
 
         for section in sections:
             if section in ('main', 'transaction', 'inst'):
-                if six.PY2:
-                    configs = dict(cnf.items(section))
-                else:
-                    configs = cnf[section]
+                configs = cnf[section]
                 if 'debug_level' in configs:
                     # logging level must be set before any other configuration option as other options might emit log
                     # messages
@@ -430,17 +459,14 @@ class AppOpticsApmConfig(object):
                         # _get_logger
                         self._set_config_value('debug_level', configs['debug_level'])
                     configs.pop('debug_level')
-                for key, val in six.iteritems(configs):
+                for key, val in configs.items():
                     self._set_config_value(section + '.' + key if section != 'main' else key, val)
                 continue
             # check for custom transaction filter configuration settings
             subsections = section.split(':')
             if len(subsections) == 3 and subsections[0] == 'transaction' and subsections[1] == 'filter':
                 # convert custom transaction filter rule into configuration dictionary
-                if six.PY2:
-                    config_dict = dict(cnf.items(section))
-                else:
-                    config_dict = cnf[section]
+                config_dict = cnf[section]
                 if not appoptics_apm.transaction_filter.add_transaction_filter_rule(config_dict):
                     util_logger.warning("Ignoring custom transaction setting [{}]".format(section))
 
@@ -456,13 +482,9 @@ class AppOpticsApmConfig(object):
         """
         if key == 'tracing_mode':
             self._set_config_value(key, value)
-            SwigContext.setTracingMode(OboeTracingMode.get_oboe_trace_mode(self.get(key)))
 
         elif key == 'sample_rate':
             self._set_config_value(key, value)
-            curr_sample_rate = self.get(key)
-            if curr_sample_rate is not None:
-                SwigContext.setDefaultSampleRate(int(curr_sample_rate * 1e6))
 
         elif key == 'log_trace_id':
             self._set_config_value(key, value)
@@ -487,10 +509,29 @@ class AppOpticsApmConfig(object):
         return value if value is not None else default
 
 
-def sys_is_traceable():
-    """Check if the required system environment variables APPOPTICS_SERVICE_KEY
-    is set."""
-    return bool(appoptics_apm.config.get('service_key'))
+class OboeReporterCodes:
+    """Return values of Oboe Reporter"""
+    OBOE_INIT_ALREADY_INIT = -1
+    OBOE_INIT_OK = 0
+    OBOE_INIT_WRONG_VERSION = 1
+    OBOE_INIT_INVALID_PROTOCOL = 2
+    OBOE_INIT_NULL_REPORTER = 3
+    OBOE_INIT_DESC_ALLOC = 4
+    OBOE_INIT_FILE_OPEN_LOG = 5
+    OBOE_INIT_UDP_NO_SUPPORT = 6
+    OBOE_INIT_UDP_OPEN = 7
+    OBOE_INIT_SSL_CONFIG_AUTH = 8
+    OBOE_INIT_SSL_LOAD_CERT = 9
+    OBOE_INIT_SSL_REPORTER_CREATE = 10
+    OBOE_INIT_SSL_MISSING_KEY = 11
+
+    @classmethod
+    def get_text_code(cls, num):
+        """Returns the textual representation of the numerical status code."""
+        for k, v in cls.__dict__.items():
+            if v == num:
+                return k
+        return None
 
 
 def appoptics_apm_init():
@@ -500,15 +541,15 @@ def appoptics_apm_init():
     3. Get sample configurations from collectors: sample_rate and sample_source"""
     global _ready
 
-    if sys_is_traceable():
-        # APPOPTICS_COLLECTOR is fetched by underlying library
-        reporter = _reporter()
-        if reporter.init_status == 0:  # OBOE_INIT_OK
-            _ready = True
-        else:
-            util_logger.error("Failed to initialize the reporter, error code={}".format(str(reporter.init_status)))
+    # APPOPTICS_COLLECTOR is fetched by underlying library
+    reporter = _reporter()
+    if reporter.init_status in (OboeReporterCodes.OBOE_INIT_OK, OboeReporterCodes.OBOE_INIT_ALREADY_INIT):
+        # If the agent was explicitly disabled, no tracing should ever happen
+        _ready = _agent_enabled
     else:
-        util_logger.error("APPOPTICS_SERVICE_KEY must be specified. Tracing disabled.")
+        util_logger.error(
+            "Failed to initialize the reporter, error code=%s (%s)" %
+            (reporter.init_status, OboeReporterCodes.get_text_code(reporter.init_status)))
 
 
 ###############################################################################
@@ -528,7 +569,7 @@ def _collect_backtraces(module_name):
     return appoptics_apm.config['inst'][module_name]['collect_backtraces']
 
 
-class Context(object):
+class Context:
     """ A wrapper around the swig Metadata """
 
     # class scope dict to manage active_instance, transaction_name, and start_time
@@ -536,7 +577,7 @@ class Context(object):
     transaction_dict = threading.local()
 
     def __init__(self, md):
-        if isinstance(md, six.string_types):
+        if isinstance(md, str):
             self._md = Metadata.fromString(md)
         else:
             self._md = md
@@ -630,13 +671,13 @@ class Context(object):
 
         ret_val = False
         if not trans_name:
-            util_logger.debug('Invalid transaction name:{n}'.format(n=_apm_str(trans_name)))
+            util_logger.debug('Invalid transaction name:{n}'.format(n=trans_name))
         elif self.is_valid():
             transaction_name_envv = os.environ.get('APPOPTICS_TRANSACTION_NAME', None)
             if transaction_name_envv:
                 util_logger.debug("Transaction name ignored as APPOPTICS_TRANSACTION_NAME environment variable is set.")
             else:
-                trans_name = _apm_str(trans_name)
+                trans_name = str(trans_name)
                 setattr(Context.transaction_dict, 'transaction_name', trans_name)
                 ret_val = True
         else:
@@ -651,7 +692,7 @@ class Context(object):
         else:
             return getattr(Context.transaction_dict, 'transaction_name', None)
 
-    def end_trace(self, event):  # Reports the last event in a trace
+    def end_trace(self, event, has_error=False):  # Reports the last event in a trace
         """Ends this trace, rendering this Context invalid.
            submit inbound metrics and get final transaction name
            update event info with transaction name
@@ -659,6 +700,7 @@ class Context(object):
            Keyword arguments:
            self -- this context instance
            event -- last trace event
+           has_error -- indicates if an error occurred in this span (only honoured in non-http spans)
         """
         http_span = getattr(Context.transaction_dict, 'http', None)
         do_metrics = getattr(Context.transaction_dict, 'do_metrics', None)
@@ -701,8 +743,6 @@ class Context(object):
                     trans_name = SwigSpan.createHttpSpan(
                         trans_name, url_tran, domain, span_time, status_code, request_method, has_error)
                 else:
-                    # error reporting is currently not used for non-http requests
-                    has_error = False
                     trans_name = SwigSpan.createSpan(trans_name, domain, span_time, has_error)
                 util_logger.debug(
                     'create span: http: {http}, transaction_name: {xn}, duration: {duration}us.'.format(
@@ -723,6 +763,7 @@ class Context(object):
                 event.add_info('URL', url_tran)
                 event.add_info('Status', getattr(Context.transaction_dict, 'status_code', 0))
             self.report(event)
+        _reporter().flush()
         self._md = None
         Context.clear_default()
 
@@ -778,7 +819,7 @@ class Context(object):
         return ''
 
 
-class Event(object):
+class Event:
     """An Event is a key/value bag that will be reported to the Tracelyzer."""
     def __init__(self, raw_evt, label, layer):
         self._evt = raw_evt
@@ -847,10 +888,10 @@ class Event(object):
             label=self.label,
         )
 
-        return _apm_str(rp)
+        return str(rp)
 
 
-class NullEvent(object):
+class NullEvent:
     """Subclass of event that will not be reported to the Tracelyzer.
 
     All methods here are no-ops. Checking for this class can be done
@@ -885,10 +926,7 @@ class NullEvent(object):
 ###############################################################################
 
 try:
-    if six.PY2:
-        from cStringIO import StringIO
-    else:
-        from io import StringIO
+    from io import StringIO
 
     import cProfile
     import pstats
@@ -931,17 +969,8 @@ def _update_event(evt, keys=None, store_backtrace=True, backtrace=None, edge_str
 def _sanitize_sql(sql_statement=""):
     """ sanitize sql statement
     """
-    # regex only works on str type, if it's python3 bytes, decode it
-    # if neither string nor bytes, cast to str type
-    # checking python3 bytes
-    is_byte_sql = not isinstance(sql_statement, str) and isinstance(sql_statement, bytes)
-
-    if is_byte_sql:
-        sql_statement = sql_statement.decode()
-    elif not isinstance(sql_statement, str):  # non-str, non-bytes type
-        sql_statement = _apm_str(sql_statement)
-
-    sql = sql_statement
+    # regex only works on str type, if it's bytes, decode it
+    sql = sql_statement.decode() if isinstance(sql_statement, bytes) else str(sql_statement)
 
     # replace '...' "..." with ? and keep '', "" intact
     # replace single quotes
@@ -957,11 +986,8 @@ def _sanitize_sql(sql_statement=""):
         r'0[1-7][0-7]*\.?[0-7]*|00\.[0-7]*)',  # oct number
         r"\g<1>0",
         sql)
-
-    if is_byte_sql:
-        sql = sql.encode()
-
-    return sql
+    # we leave it a byte string if the query was in form of a byte string before
+    return sql.encode() if isinstance(sql_statement, bytes) else sql
 
 
 def _log_event(evt, keys=None, store_backtrace=True, backtrace=None, edge_str=None):
@@ -1044,7 +1070,7 @@ def start_http_trace(
     return ctx
 
 
-def end_trace(layer, keys=None, http=False):
+def end_trace(layer, keys=None, http=False, has_error=False):
     """End a trace, reporting a final event.
 
     This will end a trace locally. If the X-Trace ID returned here is reported
@@ -1053,6 +1079,7 @@ def end_trace(layer, keys=None, http=False):
     :layer: The layer name of the final layer.
     :keys: An optional dictionary of key-value pairs to report.
     :http: A flag indicates http span for True or non for False
+    :has_error: A flag indicating if an error occurred in this span
     """
     setattr(Context.transaction_dict, 'http', http)
     if not http:
@@ -1063,7 +1090,7 @@ def end_trace(layer, keys=None, http=False):
     evt = ctx.create_event('exit', layer)
     if ctx.is_sampled():
         evt = _update_event(evt, keys=keys, store_backtrace=False)
-    ctx.end_trace(evt)
+    ctx.end_trace(evt, has_error=has_error)
 
     return evt.id()
 
@@ -1078,7 +1105,7 @@ def end_http_trace(layer, keys=None):
     :keys: An optional dictionary of key-value pairs to report.
     """
     # is this check necessary and can we move it to the end_trace() call?
-    if not isinstance(layer, six.string_types):
+    if not isinstance(layer, str):
         util_logger.debug('Layer {layer} should be a string, got {type}'.format(layer=layer, type=type(layer)))
         layer = str(layer)
     return end_trace(layer, keys, True)
@@ -1159,7 +1186,7 @@ def log_exception(msg=None, store_backtrace=True):
 
         if msg is None:
             try:
-                msg = _apm_str(val)
+                msg = str(val)
             except Exception:
                 msg = repr(val)
 
@@ -1201,7 +1228,7 @@ def set_transaction_name(trans_name=""):
 
     ctx = Context.get_default()
     ret = ctx.set_transaction_name(trans_name)
-    util_logger.debug('set_transaction_name:{n} returns  {r}'.format(n=_apm_str(trans_name), r=ret))
+    util_logger.debug('set_transaction_name:{n} returns  {r}'.format(n=trans_name, r=ret))
     return ret
 
 
@@ -1232,7 +1259,7 @@ def _function_signature(func):
         argstrings = args[:first]
         for i in range(first, len(args)):
             d = defaults[i - first]
-            if isinstance(d, six.string_types):
+            if isinstance(d, str):
                 d = "'" + d + "'"
             else:
                 d = str(d)
@@ -1279,7 +1306,7 @@ def trace(layer='Python', xtr_hdr=None, kvs=None):
     return decorate_with_trace
 
 
-class profile_block(object):
+class profile_block:
     """A context manager for AppOptics APM profiling a block of code with AppOptics APM lib.
 
     Reports an error event between entry and exit if an exception is thrown,
@@ -1522,7 +1549,7 @@ def log_method(
 
             # (optionally) report return value
             if store_return:
-                exit_kvs['ReturnValue'] = _apm_str(res)
+                exit_kvs['ReturnValue'] = str(res)
 
             # (optionally) report profiler results
             if profile and stats:
@@ -1579,6 +1606,8 @@ def _reporter():
             file_single=appoptics_apm.config.get('reporter_file_single'),
             ec2_metadata_timeout=appoptics_apm.config.get('ec2_metadata_timeout'),
             grpc_proxy=appoptics_apm.config.get('proxy'),
+            stdout_clear_nonblocking=0,
+            is_grpc_clean_hack_enabled=appoptics_apm.config.get('is_grpc_clean_hack_enabled'),
         )
 
     return reporter_instance
@@ -1586,18 +1615,23 @@ def _reporter():
 
 def _Event_addInfo_safe(func):
     def wrapped(*args, **kw):
-        try:  # call SWIG-generated Event.addInfo (from swig/oboe.py)
+        try:
             if args[1] is None:
+                # SWIG-generated Event.addInfo does not accept None types as keys, thus we need to manually convert
+                # them to a "None" string
                 args = (args[0], str(args[1]), args[2])
             return func(*args)
-        except (TypeError, IndexError):  # unrecognized type passed to addInfo SWIG binding
+        except (TypeError, IndexError):
+            # SWIG-generated Event.addInfo did either receive a wrong number of arguments, or objects of a type it can
+            # not process
             if len(args) != 3:
+                # wrong number of arguments was provided, we cannot do anything about this except logging this issue
                 util_logger.error(
                     '_Event_addInfo_safe: event.add_info provided with wrong number of arguments: Expected an event '
                     ' object, a key and value parameter, but got following arguments instead: %s.' % (args, ))
                 return False
+            # the key part of a key-value pair must be a string, attempt to convert it into one if necessary
             if not isinstance(args[1], str):
-                # Oboe only allows a string object as a key, thus try to convert it if needed
                 try:
                     args = (args[0], str(args[1]), args[2])
                 except Exception:
@@ -1606,24 +1640,23 @@ def _Event_addInfo_safe(func):
                         'convertible to a string, but object of type %s does not have a string representation.' %
                         (type(args[1])))
                     return False
-            if six.PY3 and isinstance(args[2], bytes):
-                # in Python 3 bytes type is distinct from a str type and thus needs to be converted into a str type
-                # otherwise, it would result in a output like "b'string content'"
-                args = (args[0], args[1], args[2].decode())
-            elif six.PY2 and isinstance(args[2], six.text_type):
-                # In Python 2 six.text_type represents unicode. In addition, Python 2's str() will assume ascii
-                # encoding and throw a UnicodeEncodingError if the encoding is different. Thus, we need do use the
-                # encode function directly to set encoding to utf-8 (like Python 3) and except encoding issues.
-                args = (args[0], args[1], args[2].encode('utf-8', 'replace'))
-            else:
-                # Last resort: Just try to convert the object to a string and intercept possible errors
+            # the value part of the key-value pair can only be of certain types, thus we try to convert objects of an
+            # unsupported type into a string
+            allowed_types = (int, float, bool, str)
+            if isinstance(args[2], bytes):
+                # bytes type needs to be decoded into a string otherwise, it would result in "b'string content'"
+                try:
+                    args = (args[0], args[1], args[2].decode(errors='backslashreplace'))
+                except LookupError:
+                    args = (args[0], args[1], args[2].decode(errors='ignore'))
+            elif type(args[2]) not in allowed_types:
+                # try to convert the object of unsupported type to a string and intercept possible errors
                 try:
                     args = (args[0], args[1], str(args[2]))
                 except Exception:
                     util_logger.error(
-                        '_Event_addInfo_safe: event.add_info wrong value type: value object must either be of '
-                        'one of the following types: int, float, boolean, str or have a string representation, '
-                        'but  found object of type %s.' % type(args[2]))
+                        '_Event_addInfo_safe: event.add_info wrong value type: value object must be one of %s or have a'
+                        ' string representation, but found object of type %s.' % (allowed_types, type(args[2])))
                     return False
             return func(*args)
 
@@ -1883,7 +1916,7 @@ def report_layer_init(layer='Python', keys=None):
     ctx.report_status(evt)
 
 
-class oboe_ready_code(object):
+class oboe_ready_code:
     OBOE_SERVER_RESPONSE_UNKNOWN = (0, "Oboe server : unknown error")
     OBOE_SERVER_RESPONSE_OK = (1, "Oboe server : is ready")
     OBOE_SERVER_RESPONSE_TRY_LATER = (2, "Oboe server : not ready yet, try later")
@@ -1919,7 +1952,7 @@ def appoptics_ready(wait_milliseconds=3000, integer_response=False):
 
     rc = SwigContext.isReady(wait_milliseconds)
     if not isinstance(rc, int) or not rc in oboe_ready_code.code_values():
-        util_logger.warning("Unrecognized  return code:{rc}".format(rc=_apm_str(rc)))
+        util_logger.warning("Unrecognized  return code:{rc}".format(rc=rc))
     elif rc != oboe_ready_code.OBOE_SERVER_RESPONSE_OK[0]:
         util_logger.warning(oboe_ready_code.code_values()[rc])
 
